import click
from .notes import (
    create_note_interactive,
    open_note,
    list_notes,
    delete_note,
    show_note,
    edit_note,
    tag_note,
    untag_note,
)

commands = dict(
    show=show_note,
    remove=delete_note,
    rm=delete_note,
    delete=delete_note,
    open=open_note,
    o=open_note,
    add=create_note_interactive,
    new=create_note_interactive,
    n=create_note_interactive,
    e=edit_note,
    edit=edit_note,
    t=tag_note,
    tag=tag_note,
    untag=untag_note,
)


@click.command()
@click.argument("command")
@click.argument("note_name", default=None, required=False)
@click.argument("tag", default=None, required=False)
def cli_main(command, note_name, tag):
    if command == "list" or command == "l":
        list_notes()
    else:
        if note_name is None:
            raise ValueError("No note name passed")

        try:
            if not tag:
                commands[command](note_name)
            else:
                commands[command](note_name, tag)
        except KeyError:
            raise ValueError(f"Command {command} is not recognized.")



from pyinspect import install_traceback

install_traceback()


from .note import Note
from .notes import (
    open_note,
    create_new_note,
    list_notes,
    create_note_interactive,
    delete_note,
    edit_note,
    show_note
)



import json

from .paths import note_metadata_folder

_metadata_template = dict(name="", tags=[],)


def _get_note_metadata_path(note_name, raise_error=True):
    note_name = note_name.replace(".md", "")
    if ".json" not in note_name:
        note_name += ".json"

    path = note_metadata_folder / note_name
    if not path.exists() and raise_error:
        raise FileNotFoundError(
            f"Could not find note metadata file for {note_name}"
        )
    return path


def make_note_metadata(note_name, name=None, tags=None):
    metadata = _metadata_template.copy()
    metadata["name"] = name if name is not None else ""

    if tags is not None and not isinstance(tags, list):
        tags = [tags]
    metadata["tags"] = tags if tags is not None else []

    return metadata


def save_metadata(note_name, metadata):
    path = _get_note_metadata_path(note_name, raise_error=False)
    with open(path, "w") as jfile:
        json.dump(metadata, jfile)


def get_note_metadata(note_name):
    path = _get_note_metadata_path(note_name)
    with open(path, "r") as jfile:
        meta = json.load(jfile)
    return meta



from rich.markdown import Markdown
from rich import print
from rich.panel import Panel
from pyinspect import Report
from random import choice
from pyinspect._colors import (
    lightred,
    orange,
    lightorange,
    green,
    dimgreen,
    lightgreen2,
    gray,
    lilla,
    yellow,
    salmon,
    mocassin,
)

from ._notes import _get_note_path, note_editor
from ._metadata import make_note_metadata, get_note_metadata, save_metadata

blackboard = "#1f1f1f"
colors = [
    lightred,
    orange,
    lightorange,
    green,
    dimgreen,
    lightgreen2,
    gray,
    lilla,
    yellow,
    salmon,
    mocassin,
]


class Note:
    def __init__(self, note_name, raise_error=True):
        self.path = _get_note_path(note_name, raise_error=raise_error)
        self.name = note_name

        try:
            self._get_content()
        except FileNotFoundError as e:
            if raise_error:
                raise FileNotFoundError(e)

    def _get_content(self):
        with open(self.path, "r") as f:
            self.raw_content = f.read()
        self.content = Markdown(self.raw_content, inline_code_lexer='python')
        self.metadata = get_note_metadata(self.name)

    @property
    def tags(self):
        return self.metadata["tags"]

    @property
    def tags_render(self):
        tags = self.metadata["tags"]
        if not tags:
            return [
                Panel.fit("no tags", border_style=blackboard, padding=(-1, 1))
            ]
        else:
            tt = []
            for tag in tags:
                tt.append(
                    Panel.fit(
                        tag, border_style=choice(colors), padding=(-1, 1)
                    )
                )

            return tt

    def add_tag(self, *tags):
        for tag in tags:
            if not isinstance(tag, str):
                raise TypeError("tag must be a string")

            if tag not in self.metadata["tags"]:
                self.metadata["tags"].append(tag)

    def pop_tag(self, *tags):
        for tag in tags:
            if tag in self.metadata["tags"]:
                idx = self.metadata["tags"].index(tag)
                self.metadata["tags"].pop(idx)

    @classmethod
    def from_string(cls, string, name):
        note = cls(name, raise_error=False)
        note.raw_content = string

        # make empty metadata
        note.metadata = make_note_metadata(name)

        return note

    def save(self):
        with open(self.path, "w") as out:
            out.write(self.raw_content)
        save_metadata(self.name, self.metadata)

        print(f"[{mocassin}]:ok_hand:  Saved note as: [{orange}]{self.path.name}")

    def edit(self):
        note_editor(self.path)
        self._get_content()

    def show(self):
        show = Report(
            title=f":pencil:  {self.name}",
            show_info=True,
            color=mocassin,
            accent=orange,
        )
        show._type = self.name

        show.add(self.content, 'rich')

        show.print()
 


from datetime import datetime


def format_timestamp(timestamp):
    date = datetime.fromtimestamp(timestamp)

    if date.date() == datetime.today().date():
        return date.strftime("%H:%M:%S")
    else:
        return date.strftime("%d/%m/%Y")



from donote.notes import (
    create_new_note,
    open_note,
    list_notes,
    delete_note,
    show_note,
)
from donote.note import Note


def test_create_note():
    create_new_note("testing")


def test_open_note():
    note = open_note("testing")
    if not isinstance(note, Note):
        raise AssertionError


def test_note_attributes():
    note = open_note("testing")

    note.name == "testing"
    note.raw_content = ""


def test_tags():
    note = open_note("testing")
    assert note.tags == []

    note.add_tag("test")
    assert note.tags == ["test"]

    note.pop_tag("test")
    assert note.tags == []


def test_note_save():
    note = open_note("testing")
    note.save()


def test_note_show():
    note = open_note("testing")
    note.show()

    show_note("testing")


def test_list_notes():
    list_notes()


def test_delete_note():
    delete_note("testing", force=True)



from rich.filesize import decimal as format_size
import sys
import tempfile
import subprocess
import os
from pathlib import Path
from rich import print
from rich.panel import Panel


from .paths import notes_folder, windows
from .utils import format_timestamp


def _get_note_path(note_name, raise_error=True):
    if ".md" not in note_name:
        note_name += ".md"
    path = notes_folder / note_name
    if not path.exists() and raise_error:
        raise FileNotFoundError(f"Could not find note with name {note_name}")
    return path


def note_editor(file_path=None):
    # Create temp file to write note
    if file_path is None:
        temp = tempfile.NamedTemporaryFile(mode="w+t", delete=False)
        file_path = temp.name
        cleanup = True
    else:
        file_path = Path(file_path)
        cleanup = False
        if not file_path.exists():
            raise FileNotFoundError(
                f"Could not open note editor, file not found: {file_path}"
            )

    try:
        if not windows:
            subprocess.call(["nano", file_path])
        else:
            subprocess.call(
                ["C:\\Program Files\\Git\\usr\\bin\\nano.exe", file_path]
            )
    except Exception as e:
        print(f"[red]Failed to open note editor: {e}")
        return None

    # Read temp content and close
    with open(file_path) as f:
        content = f.read()

    if cleanup:
        temp.close()
        os.unlink(temp.name)

    return content


def get_note_file_metadata(note_name):
    stats = _get_note_path(note_name).stat()

    if sys.platform == "Win32":
        created = format_timestamp(stats.st_ctime)
    else:
        try:
            created = format_timestamp(stats.st_birthtime)
        except AttributeError:
            # no easy way to do this in Linux
            created = ""

    edited = format_timestamp(stats.st_mtime)

    size = format_size(stats.st_size)

    num_lines = str(sum(1 for line in open(_get_note_path(note_name))))

    items = []
    for elem in (note_name, created, edited, size, num_lines):
        items.append(
            Panel.fit(
                elem,
                border_style="#1f1f1f",  # choice(colors),
                padding=(-1, 1),
            )
        )
    return items



from setuptools import setup, find_namespace_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, "README.md"), encoding="utf-8") as f:
    long_description = f.read()

requirements = ["pyinspect", "click", "rich>=9.0.0"]

setup(
    name="donote",
    version="0.0.3",
    description="donote",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=requirements,
    extras_require={"dev": ["coverage-badge", "click"]},
    python_requires=">=3.6,",
    packages=find_namespace_packages(),
    include_package_data=True,
    url="https://github.com/FedeClaudi/knowledge_base",
    author="Federico Claudi",
    zip_safe=False,
    entry_points={"console_scripts": ["dono = donote.cli:cli_main"]},
)



import sys
from pathlib import Path

if sys.platform == "win32":
    windows = True
    base_folder = Path("D:\\Dropbox (UCL - SWC)\\Rotation_vte\\knowledge_base")
else:
    windows = False
    base_folder = Path(
        "/Users/federicoclaudi/Dropbox (UCL)/Rotation_vte/knowledge_base"
    )

# Make subdirs
notes_folder = base_folder / "notes"
note_metadata_folder = base_folder / ".notes_metadata"

notes_folder.mkdir(exist_ok=True)
note_metadata_folder.mkdir(exist_ok=True)



from pyinspect._colors import mocassin, orange
from rich.table import Table
from rich.box import SIMPLE_HEAVY
from rich.prompt import Confirm
from rich import print
from pyinspect.utils import dir_files

from ._notes import (
    get_note_file_metadata,
    _get_note_path,
    note_editor,
)
from .note import Note
from .paths import notes_folder
from ._metadata import _get_note_metadata_path


def open_note(note_name):
    try:
        return Note(note_name)
    except Exception:
        if Confirm.ask("No note found, create a new one?", default=True):
            create_note_interactive(note_name)


def show_note(note_name):
    Note(note_name).show()


def edit_note(note_name):
    Note(note_name).edit()
    print(f':pencil:  [{mocassin}]finished editing [{orange}]{note_name}')


def tag_note(note_name, tag):
    note = Note(note_name)
    note.add_tag(tag)
    note.save()
    print(f':ok_hand:  [{mocassin}]added tag to [{orange}]{note_name}')


def untag_note(note_name, tag):
    note = Note(note_name)
    note.pop_tag(tag)
    note.save()
    print(f':ok_hand:  [{mocassin}]removed tag from [{orange}]{note_name}')


def get_all_notes():
    notes = dir_files(notes_folder, pattern="*.md")
    return [open_note(n.name) for n in notes]


def create_note_interactive(note_name):
    print(f"[{mocassin}]Making a new note: [{orange}]{note_name}")

    path = _get_note_path(note_name, raise_error=False)
    if path.exists():
        print(f"[{orange}]A note with name {note_name} already exists.")
        if not Confirm.ask(f"Overwrite {path.name}?", default=False):
            print("Okay, not overwriting")
            return

    # Create a live editor to fill in note content
    try:
        content = note_editor()
    except Exception:
        print(f"[{orange}]Creating empty note instead")
        return create_new_note(note_name)

    # Save note to file
    Note.from_string(content, note_name).save()


def create_new_note(note_name):
    print(f"[{mocassin}]Making a new empty note: [{orange}]{note_name}")

    path = _get_note_path(note_name, raise_error=False)
    if path.exists():
        print(f"[{orange}]A note with name {note_name} already exists.")
        if not Confirm.ask(f"Overwrite {path.name}?", default=False):
            print(f"[{mocassin}]    okay, not saving then.")
            return

    note = Note.from_string("", note_name)
    note.save()

    print(f"[{mocassin}]    saved note at: [{orange}]{note_name}")
    return note


def delete_note(note_name, force=False):
    path = _get_note_path(note_name)
    metadata_path = _get_note_metadata_path(note_name)

    if not force:
        confirm = Confirm.ask(
            f"[{mocassin}]Deleting note: [{orange}]{note_name}[/{orange}], continue?",
            default=False,
        )

    if force or confirm:
        path.unlink()
        metadata_path.unlink()
        print(f"    [{mocassin}]removed: [{orange}]{note_name}")


def list_notes():

    tb = Table(header_style="bold green", box=SIMPLE_HEAVY, min_width=88)
    tb.add_column(header="Name", min_width=20)
    tb.add_column(header="# lines", justify="center", min_width=8)
    tb.add_column(header="size", justify="center", min_width=16)
    tb.add_column(header="edited", justify="center", min_width=16)
    tb.add_column(header="created", style="dim", justify="center", min_width=16)
    tb.add_column(header="tags")

    notes = get_all_notes()
    for note in notes:
        name, created, edited, size, num_lines = get_note_file_metadata(
            note.name
        )
        name.style = f"bold {mocassin}"

        tb.add_row(name, num_lines, size, edited, created, *note.tags_render)
    print(tb)
